#********************************************************************************
# Administration of qemu virtual machines to use with qemu-init                 *
#                                                                               *
#  Author: Kai Peter, ©2016-                                                    *
# Version: 0.30                                                                 *
# License: ISC                                                                  *
#                                                                               *
# Description: register/unregister virtual machines and show some info          *
#********************************************************************************
CFF="REGFILE"
PATH=`dirname $0`:$PATH

showHelp() {
  echo -e "Usage: qemu-adm  [-r|-s|-u [VM] ] [ -l ] [ -h ]"
  echo -e ""
  echo -e "Options:"
  echo -e "   -a <VM>\tchange (revert) the AUTOSTART option"
  echo -e "   -h\t\tshow this help screen"
  echo -e "   -l\t\tshow registered virtual machines"
  echo -e "   -n <VM>\tcreate a config file and rc link"
  echo -e "   -p <VM>\tshow vnc and monitor configuration (ports) from qemu monitor"
  echo -e "   -r <VM>\tregister a virtual machine"
  echo -e "   -u <VM>\tunregister a virtual machine"
  echo -e "   -s\t\tshow runtime configuration of virtual machines"
  echo -e "   -y\t\tcreate a rc file - only together with -n"
  echo -e ""
}

error() {
  case $1 in
    1) echo "Invalid directory!";;
    2) echo "Couldn't find template file!";;
  esac
  exit 1
}

showRuntimeConf() {    # depracated, use 'qemu-init shortname config' instead
  VMRC=$(echo `basename $VMRC` | cut -d\- -f1)   # use the 'short' name
  QEMUBIN=$(which qemu-system-`uname -m`)
  TMP=`ps ax | grep $VMRC | grep qemu-system | sed 's/ -/;/g'`
  [ ! "$TMP" ] && exit 1
  printf "\033[1m$VMRC is running with the following options:\033[0m\n"
  echo $QEMUBIN
  echo $TMP | awk -F";" '{for (i=2;i<=NF;i++) printf("\t-%s\n", $i)}' <<< "$TMP"
}

list() {
  echo "Registered VM's:"
  [ -f "$CFF" ] && cat "$CFF" | grep -v '^#' | grep 'qemu.rc'
}

validate() {
  # validate the .rc file - it HAVE TO be a link!
  [ -L $VMRC-qemu.rc ] && VMRC=$VMRC-qemu.rc && echo 2
  cd $(dirname $VMRC)   # get the full path
  VMRC=$PWD/`basename $VMRC`
  cd $OLDPWD
  [ ! -L $VMRC ] && echo "error: invalid: $VMRC" && exit 1
  return
}

registerVM() {
  # the full path is required to register
  validate
  grep "$VMRC" "$CFF" 2>&1 >/dev/null
  [ $? = 0 ] && echo "$VMRC is registered already!" && exit 1
  echo "$VMRC:OFF::" >> "$CFF"
  [ $? = 0 ] && echo "Successful registered $VMRC"
}

unregisterVM() {
  NAMEOK=0
  # check for qualified short name
  # first we check if name is with a path ...
  echo $VMRC | grep ^/ | grep -qemu.rc >/dev/null
  EC=$? ; [ $EC = 0 ] && NAMEOK=1
  # ... and if not, second we check for a valid short name
  [ ! $NAMEOK = 1 ] && grep /$VMRC-qemu.rc "$CFF" 2>&1 >/dev/null
  EC=$? ; [ $EC = 0 ] && NAMEOK=1
  # last, we check if it's in REGFILE
  [ $NAMEOK = 1 ] && grep $VMRC "$CFF" >/dev/null
  EC=$?
  [ ! $EC = 0 ] && printf "\033[1mNot a regitered VM: $VMRC\033[0m\n" && exit 1

  cat "$CFF" | grep -v $VMRC > "$CFF.tmp"
  [ $? = 0 ] && mv "$CFF.tmp" "$CFF" && \
  echo "$VMRC successful unregistered." && exit 0
}

createConfig() {
  TMP=`basename $VMRC 2>/dev/null`
  DIR=`dirname $VMRC 2>/dev/null`
  TPL="TEMPLATE"
  [ "$DIR" ] && [ -d "$DIR" ] && cd "$DIR" || error 1
  [ -f "$TPL" ] || error 2
  cp -n "$TPL" $TMP-qemu.cfg
  QEMUINIT=`which qemu-init`
#  # convention: '<name>-qemu.cfg' have to exist in the actual folder
#  [ ! -f $TMP-qemu.cfg ] && echo "Couldn't find $VMRC-qemu.cfg!" && exit 1
  [ "$MKRCLINK" = 1 ] && ln -sf $QEMUINIT $TMP-qemu.rc
}

autostart() {

#  echo O: $O   V: $VMRC
  # shortname only - yet !!
  grep "$VMRC"-qemu.rc "$CFF" >/dev/null
  EC=$?
  [ ! $EC = 0 ] && printf "\033[1mNot a regitered VM: $VMRC\033[0m\n" && exit 1
  while IFS= read L ; do
    echo "$L" | grep $VMRC > /dev/null
    if [ $? = 0 ] ; then
      F1=`echo "$L" | cut -d: -f1`
      F2=`echo "$L" | cut -d: -f2`
      F2=`echo "$F2" | tr '[A-Z]' '[a-z]'`
      [ "$F2" = "on"  ] && F2=1
      [ "$F2" = "yes" ] && F2=1
      [ "$F2" = "y"   ] && F2=1
      [ "$F2" = "1"   ] && F2=1
      [ "$F2" = "1" ] && F2=OFF || F2=ON
      L="$F1:$F2:::"
    fi
    echo "$L" >> "$CFF".tmp
  done < "$CFF"
  mv "$CFF.tmp" "$CFF"
  echo "Switched AUTOSTART of $VMRC to: $F2"
}

ports() {
  printf "\033[1mStatus VNC:\033[0m\n"
  qemu-cmd "$VMRC" "info vnc"
  printf "\n\033[1mStatus Monitor:\033[0m\n"
  qemu-cmd "$VMRC" "info chardev" | grep monitor | sort
}

[ ! "$1" ] && showHelp && exit 1  # don't run w/o any parameter
while getopts a:hln:p:r:u:s:y o; do
    case "${o}" in
        a) VMRC=($OPTARG) ; ACTION="autostart";;
        h) showHelp ; exit 0;;
        l) ACTION="list";;                         		# better: -s
        n) VMRC=($OPTARG) ; ACTION="createConfig";;  	# new
        p) VMRC=($OPTARG) ; ACTION="ports";;
        r) VMRC=($OPTARG) ; ACTION="registerVM";;
        s) VMRC=($OPTARG) ; ACTION="showRuntimeConf";;	# better: ??
        u) VMRC=($OPTARG) ; ACTION="unregisterVM";;
        y) MKRCLINK=1;;                         # only with '-n'
#        z) VMRC=($OPTARG) ; createConfig;;
        *) echo Invalid option! ; showHelp ; exit 1;;
    esac
done ; shift $((OPTIND-1))

[ "$ACTION" ] && "$ACTION" && ACTION="" && exit 0
echo Invalid option! ; showHelp ; exit 1
